# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/static_data/v1/set_static_data.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.type import datetime_pb2 as google_dot_type_dot_datetime__pb2
from google.type import date_pb2 as google_dot_type_dot_date__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from systemathics.apis.services.static_data.v1 import static_data_pb2 as systemathics_dot_apis_dot_services_dot_static__data_dot_v1_dot_static__data__pb2
from systemathics.apis.type.shared.v1 import asset_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_asset__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n?systemathics/apis/services/static_data/v1/set_static_data.proto\x12-systemathics.apis.services.set_static_data.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x1agoogle/type/datetime.proto\x1a\x16google/type/date.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a;systemathics/apis/services/static_data/v1/static_data.proto\x1a,systemathics/apis/type/shared/v1/asset.proto\"l\n\x0e\x41ssetIdRequest\x12\x0e\n\x02id\x18\x01 \x01(\x03R\x02id\x12J\n\nasset_type\x18\x02 \x01(\x0e\x32+.systemathics.apis.type.shared.v1.AssetTypeR\tassetType\"\xa5\x01\n\x12\x43reateAssetRequest\x12-\n\x08validity\x18\x01 \x01(\x0b\x32\x11.google.type.DateR\x08validity\x12W\n\tcds_index\x18\x02 \x01(\x0b\x32\x38.systemathics.apis.services.static_data.v1.CdsIndexEntryH\x00R\x08\x63\x64sIndexB\x07\n\x05\x61sset\"\xa8\x01\n\x05\x41sset\x12\x0e\n\x02id\x18\x01 \x01(\x03R\x02id\x12-\n\x08validity\x18\x02 \x01(\x0b\x32\x11.google.type.DateR\x08validity\x12W\n\tcds_index\x18\x03 \x01(\x0b\x32\x38.systemathics.apis.services.static_data.v1.CdsIndexEntryH\x00R\x08\x63\x64sIndexB\x07\n\x05\x61sset2\x99\x06\n\x14SetStaticDataService\x12\xa1\x01\n\x06\x43reate\x12\x41.systemathics.apis.services.set_static_data.v1.CreateAssetRequest\x1a\x34.systemathics.apis.services.set_static_data.v1.Asset\"\x1e\x82\xd3\xe4\x93\x02\x18\"\x16/v1/staticdata/{asset}\x12\xa2\x01\n\x06GetOne\x12=.systemathics.apis.services.set_static_data.v1.AssetIdRequest\x1a\x34.systemathics.apis.services.set_static_data.v1.Asset\"#\x82\xd3\xe4\x93\x02\x1d\x12\x1b/v1/staticdata/{asset}/{id}\x12\x94\x01\n\x06GetAll\x12\x32.systemathics.apis.type.shared.v1.AssetTypeRequest\x1a\x34.systemathics.apis.services.set_static_data.v1.Asset\"\x1e\x82\xd3\xe4\x93\x02\x18\x12\x16/v1/staticdata/{asset}0\x01\x12\x99\x01\n\x06Update\x12\x34.systemathics.apis.services.set_static_data.v1.Asset\x1a\x34.systemathics.apis.services.set_static_data.v1.Asset\"#\x82\xd3\xe4\x93\x02\x1d\x1a\x1b/v1/staticdata/{asset}/{id}\x12\x84\x01\n\x06\x44\x65lete\x12=.systemathics.apis.services.set_static_data.v1.AssetIdRequest\x1a\x16.google.protobuf.Empty\"#\x82\xd3\xe4\x93\x02\x1d*\x1b/v1/staticdata/{asset}/{id}B\x98\x02\n1com.systemathics.apis.services.set_static_data.v1B\x12SetStaticDataProtoP\x01\xa2\x02\x04SASS\xaa\x02+Systemathics.Apis.Services.SetStaticData.V1\xca\x02+Systemathics\\Apis\\Services\\SetStaticData\\V1\xe2\x02\x37Systemathics\\Apis\\Services\\SetStaticData\\V1\\GPBMetadata\xea\x02/Systemathics::Apis::Services::SetStaticData::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'systemathics.apis.services.static_data.v1.set_static_data_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n1com.systemathics.apis.services.set_static_data.v1B\022SetStaticDataProtoP\001\242\002\004SASS\252\002+Systemathics.Apis.Services.SetStaticData.V1\312\002+Systemathics\\Apis\\Services\\SetStaticData\\V1\342\0027Systemathics\\Apis\\Services\\SetStaticData\\V1\\GPBMetadata\352\002/Systemathics::Apis::Services::SetStaticData::V1'
  _globals['_SETSTATICDATASERVICE'].methods_by_name['Create']._options = None
  _globals['_SETSTATICDATASERVICE'].methods_by_name['Create']._serialized_options = b'\202\323\344\223\002\030\"\026/v1/staticdata/{asset}'
  _globals['_SETSTATICDATASERVICE'].methods_by_name['GetOne']._options = None
  _globals['_SETSTATICDATASERVICE'].methods_by_name['GetOne']._serialized_options = b'\202\323\344\223\002\035\022\033/v1/staticdata/{asset}/{id}'
  _globals['_SETSTATICDATASERVICE'].methods_by_name['GetAll']._options = None
  _globals['_SETSTATICDATASERVICE'].methods_by_name['GetAll']._serialized_options = b'\202\323\344\223\002\030\022\026/v1/staticdata/{asset}'
  _globals['_SETSTATICDATASERVICE'].methods_by_name['Update']._options = None
  _globals['_SETSTATICDATASERVICE'].methods_by_name['Update']._serialized_options = b'\202\323\344\223\002\035\032\033/v1/staticdata/{asset}/{id}'
  _globals['_SETSTATICDATASERVICE'].methods_by_name['Delete']._options = None
  _globals['_SETSTATICDATASERVICE'].methods_by_name['Delete']._serialized_options = b'\202\323\344\223\002\035*\033/v1/staticdata/{asset}/{id}'
  _globals['_ASSETIDREQUEST']._serialized_start=364
  _globals['_ASSETIDREQUEST']._serialized_end=472
  _globals['_CREATEASSETREQUEST']._serialized_start=475
  _globals['_CREATEASSETREQUEST']._serialized_end=640
  _globals['_ASSET']._serialized_start=643
  _globals['_ASSET']._serialized_end=811
  _globals['_SETSTATICDATASERVICE']._serialized_start=814
  _globals['_SETSTATICDATASERVICE']._serialized_end=1607
# @@protoc_insertion_point(module_scope)



"""Systemathics Ganymede  API Token Helpers

This module helps to create tokens to access Systemathics Ganymede authenticated API.

functions:
    get_cds_index_daily - Get CDS Index daily data as a DataFrame using Ganymede gRPC API.
    get_cds_index_intraday - Get CDS Index intraday data as a DataFrame using Ganymede gRPC API.
    get_cds_daily - Get CDS daily data as a DataFrame using Ganymede gRPC API.
    get_cds_intraday - Get CDS intraday data as a DataFrame using Ganymede gRPC API.
    get_index_tick - Get Index tick data as a DataFrame using Ganymede gRPC API.
    get_future_daily - Get future daily data as a DataFrame using Ganymede gRPC API.
    get_equity_daily - Get equity daily data as a DataFrame using Ganymede gRPC API.
"""


import grpc
import pandas as pd
from datetime import date,datetime
from google.type import date_pb2
from google.type import datetime_pb2
from google.type import timeofday_pb2


from systemathics.apis.type.shared.v1 import asset_pb2 as asset
from systemathics.apis.type.shared.v1 import constraints_pb2 as constraints
from systemathics.apis.type.shared.v1 import date_interval_pb2 as date_interval
import systemathics.apis.type.shared.v1.sampling_pb2 as sampling
import systemathics.apis.type.shared.v1.identifier_pb2 as identifier
import systemathics.apis.services.daily.v1.daily_bars_pb2 as daily_bars
import systemathics.apis.services.daily.v1.daily_bars_pb2_grpc as daily_bars_service
import systemathics.apis.services.daily.v2.get_daily_pb2 as get_daily
import systemathics.apis.services.daily.v2.get_daily_pb2_grpc as get_daily_service
import systemathics.apis.services.intraday.v2.get_intraday_pb2 as get_intraday
import systemathics.apis.services.intraday.v2.get_intraday_pb2_grpc as get_intraday_service
import systemathics.apis.services.tick.v2.get_tick_pb2 as get_tick
import systemathics.apis.services.tick.v2.get_tick_pb2_grpc as get_tick_service
from systemathics.apis.type.shared.v1 import time_interval_pb2 as time_interval

import systemathics.apis.helpers.token_helpers as token_helpers
import systemathics.apis.helpers.channel_helpers as channel_helpers

def get_cds_index_daily(ticker, start_date=None, end_date=None, batch=None, selected_fields=None, provider="Markit"):
    """
    Fetch CDS Index daily data from gRPC API for a given ticker and date range.
    
    Parameters:
    ticker (str): The ticker symbol
    start_date (datetime.date or str, optional): Start date for data retrieval. 
                                                 If None, set not limits
    end_date (datetime.date or str, optional): End date for data retrieval.
                                               If None, uses today's date
    batch (str, optional): Batch name to be used for filtering. If None, gets all batches.
    selected_fields (list, optional): List of specific fields to retrieve. If None, gets all fields.
    provider (str): Data provider, default is "Markit"
    
    Returns:
    pd.DataFrame: DataFrame with Date as index and all available fields as columns
    """
    

    
    # All available fields
    all_fields = [
        "CompositePriceAsk", "CompositePriceBid", "CompositeSpreadAsk",
        "CompositeSpreadBid", "ConventionalSpread", "CreditDv01",
        "DefaultProbability", "Heat", "IrDv01", "JumpToDefault",
        "JumpToZero", "ModelPrice", "ModelSpread", "Price",
        "Rec01", "RiskyPv01", "SkewPrice", "SkewSpread"
    ]
    
    # Use all fields if none specified, otherwise validate selected fields
    if selected_fields is None:
        fields = all_fields
    else:
        fields = [f for f in selected_fields if f in all_fields]
        if not fields:
            raise ValueError("No valid fields selected")
        
    # Create identifier
    id = identifier.Identifier(
        asset_type=asset.AssetType.ASSET_TYPE_CDS_INDEX,
        ticker=ticker
    )
    id.provider.value = provider
    
    # Build constraints only if we have at least one date
    constraints_obj = None
    if start_date is not None or end_date is not None:
        # Create DateInterval with only the dates that are provided
        date_interval_kwargs = {}
        if start_date is not None:
            date_interval_kwargs['start_date'] = _parse_date_input(start_date)
        if end_date is not None:
            date_interval_kwargs['end_date'] = _parse_date_input(end_date)
        
        constraints_obj = constraints.Constraints(
            date_intervals=[date_interval.DateInterval(**date_interval_kwargs)]
        )
    
    if batch is None:
        # Create request with or without constraints
        request_kwargs = {
            'identifier': id,
            'fields': fields
        }
        if constraints_obj is not None:
            request_kwargs['constraints'] = constraints_obj
    
    try:
        # Open gRPC channel
        with channel_helpers.get_grpc_channel() as channel:
            # Send request and receive response
            token = token_helpers.get_token()
            first = True
            response = []
            info = None
            # Create service stub
            service = get_daily_service.DailyServiceStub(channel)
            

            if batch is None:
                          # Create request with or without constraints
                request_kwargs = {
                    'identifier': id,
                    'fields': fields
                }
                if constraints_obj is not None:
                    request_kwargs['constraints'] = constraints_obj
                
                vector_request = get_daily.DailyRequest(**request_kwargs)
                
                for data in service.DailyVectorStream(
                    request=vector_request, 
                    metadata=[('authorization', token)] 
                ):
                    if first:
                        info = data
                        first = False
                    else:
                        response.append(data.data)
            
            else:
                
                request_kwargs = {
                    'identifier': id,
                    'fields': fields,
                    'key': batch
                }
                if constraints_obj is not None:
                    request_kwargs['constraints'] = constraints_obj
                    
                vector_key_request = get_daily.DailyVectorKeyRequest(**request_kwargs)
                
                for data in service.DailyVectorKeyStream(
                    request=vector_key_request, 
                    metadata=[('authorization', token)]
                ):
                    if first:
                        info = data 
                        first = False
                    else:
                        response.append(data.data)

        # Process the response
        if not response or info is None:
            print("No data received")
            return pd.DataFrame()
        
        # Get field indices
        available_fields = [f for f in info.info.fields]
        field_indices = {field: available_fields.index(field) 
                        for field in fields if field in available_fields}
        
        # Extract dates
        dates = [date(d.date.year, d.date.month, d.date.day) for d in response]
        
        # Extract keys
        keys = [b.key for b in response]
        
        # Create dictionary for DataFrame
        data_dict = {'Key': keys}
        
        # Extract data for each field
        for field_name, field_index in field_indices.items():
            data_dict[field_name] = [b.data[field_index] for b in response]
        
        # Create DataFrame
        df = pd.DataFrame(data_dict, index=dates)
        df.index.name = 'Date'
        
        # Sort by date for better readability
        df = df.sort_index()
    
        
        return df
        
    except grpc.RpcError as e:
        print(f"gRPC Error: {e.code().name}")
        print(f"Details: {e.details()}")
        return pd.DataFrame()
    except Exception as e:
        print(f"Error: {str(e)}")
        return pd.DataFrame()

def get_cds_daily(ticker, start_date=None, end_date=None, batch=None, selected_fields=None, provider="Markit"):
    """
    Fetch CDS daily data from gRPC API for a given ticker and date range.
    
    Parameters:
    ticker (str): The ticker symbol
    start_date (datetime.date or str, optional): Start date for data retrieval. 
                                                 If None, set not limits
    end_date (datetime.date or str, optional): End date for data retrieval.
                                               If None, uses today's date
    batch (str, optional): Batch name to be used for filtering. If None, gets all batches.
    selected_fields (list, optional): List of specific fields to retrieve. If None, gets all fields.
    provider (str): Data provider, default is "Markit"
    
    Returns:
    pd.DataFrame: DataFrame with Date as index and all available fields as columns
    """
    

    
    # All available fields for individual CDS
    all_fields = [
        "ConventionalSpread",
        "ParSpread",
        "Upfront",
        "RealRecovery",
        "AssumedRecovery",
        "DefaultProbability",
        "JumpToDefault",
        "JumpToZero"
    ]
    
    # Use all fields if none specified, otherwise validate selected fields
    if selected_fields is None:
        fields = all_fields
    else:
        fields = [f for f in selected_fields if f in all_fields]
        if not fields:
            raise ValueError("No valid fields selected")
        
    # Create identifier for individual CDS (not index)
    id = identifier.Identifier(
        asset_type=asset.AssetType.ASSET_TYPE_CDS,
        ticker=ticker
    )
    id.provider.value = provider
    
    # Build constraints only if we have at least one date
    constraints_obj = None
    if start_date is not None or end_date is not None:
        # Create DateInterval with only the dates that are provided
        date_interval_kwargs = {}
        if start_date is not None:
            date_interval_kwargs['start_date'] = _parse_date_input(start_date)
        if end_date is not None:
            date_interval_kwargs['end_date'] = _parse_date_input(end_date)
        
        constraints_obj = constraints.Constraints(
            date_intervals=[date_interval.DateInterval(**date_interval_kwargs)]
        )
    
    if batch is None:
        # Create request with or without constraints
        request_kwargs = {
            'identifier': id,
            'fields': fields
        }
        if constraints_obj is not None:
            request_kwargs['constraints'] = constraints_obj
    
    try:
        # Open gRPC channel
        with channel_helpers.get_grpc_channel() as channel:
            # Send request and receive response
            token = token_helpers.get_token()
            first = True
            response = []
            info = None
            # Create service stub
            service = get_daily_service.DailyServiceStub(channel)
            

            if batch is None:
                          # Create request with or without constraints
                request_kwargs = {
                    'identifier': id,
                    'fields': fields
                }
                if constraints_obj is not None:
                    request_kwargs['constraints'] = constraints_obj
                
                vector_request = get_daily.DailyRequest(**request_kwargs)
                
                for data in service.DailyVectorStream(
                    request=vector_request, 
                    metadata=[('authorization', token)] 
                ):
                    if first:
                        info = data
                        first = False
                    else:
                        response.append(data.data)
            
            else:
                
                request_kwargs = {
                    'identifier': id,
                    'fields': fields,
                    'key': batch
                }
                if constraints_obj is not None:
                    request_kwargs['constraints'] = constraints_obj
                    
                vector_key_request = get_daily.DailyVectorKeyRequest(**request_kwargs)
                
                for data in service.DailyVectorKeyStream(
                    request=vector_key_request, 
                    metadata=[('authorization', token)]
                ):
                    if first:
                        info = data 
                        first = False
                    else:
                        response.append(data.data)

        # Process the response
        if not response or info is None:
            print("No data received")
            return pd.DataFrame()
        
        # Get field indices
        available_fields = [f for f in info.info.fields]
        field_indices = {field: available_fields.index(field) 
                        for field in fields if field in available_fields}
        
        # Extract dates
        dates = [date(d.date.year, d.date.month, d.date.day) for d in response]
        
        # Extract keys
        keys = [b.key for b in response]
        
        # Create dictionary for DataFrame
        data_dict = {'Key': keys}
        
        # Extract data for each field
        for field_name, field_index in field_indices.items():
            data_dict[field_name] = [b.data[field_index] for b in response]
        
        # Create DataFrame
        df = pd.DataFrame(data_dict, index=dates)
        df.index.name = 'Date'
        
        # Sort by date for better readability
        df = df.sort_index()
    
        
        return df
        
    except grpc.RpcError as e:
        print(f"gRPC Error: {e.code().name}")
        print(f"Details: {e.details()}")
        return pd.DataFrame()
    except Exception as e:
        print(f"Error: {str(e)}")
        return pd.DataFrame()

def get_index_tick(ticker, start_date=None, end_date=None, start_time=None, end_time=None, selected_fields=None, provider="GoldmanSachs"):
    """
    Fetch Index tick data from gRPC API for a given ticker and date range with optional client-side time filtering.    
    
    Parameters:
    ticker (str): The ticker symbol
    start_date (datetime.date or str, optional): Start date for data retrieval.
                                                 If None, set no limits
    end_date (datetime.date or str, optional): End date for data retrieval.
                                               If None, set no limits
    start_time (str, optional): Start time in 'HH:MM' format (e.g., '09:30') or 'HH:MM:ss' format (e.g., '09:30:05') or for client-side filtering.
                                If None, no time restriction
    end_time (str, optional): End time in 'HH:MM' format (e.g., '16:00') or 'HH:MM:ss' format (e.g., '16:25:45')for client-side filtering.
                              If None, no time restriction
    selected_fields (list, optional): List of specific fields to retrieve.
                                      If None, gets all fields.
    provider (str): Data provider, default is "GoldmanSachs"
    
    Returns:
    pd.DataFrame: DataFrame with Datetime as index and all available fields as columns
    """

    # All available fields for Index tick data
    all_fields = [
        "AskBenchmarkSpread",
        "AskCleanPrice", 
        "AskDirtyPrice",
        "AskGSpread",
        "AskModifiedDuration",
        "AskYield",
        "AskZSpread",
        "BidBenchmarkSpread",
        "BidCleanPrice",
        "BidDirtyPrice", 
        "BidGSpread",
        "BidModifiedDuration",
        "BidYield",
        "BidZSpread",
        "MidBenchmarkSpread",
        "MidCleanPrice",
        "MidDirtyPrice",
        "MidGSpread", 
        "MidModifiedDuration",
        "MidYield",
        "MidZSpread",
        "OfficialBenchmarkSpread",
        "OfficialCleanPrice",
        "OfficialDirtyPrice",
        "OfficialGSpread",
        "OfficialModifiedDuration", 
        "OfficialYield",
        "OfficialZSpread"
    ]    
    
    # Use all fields if none specified, otherwise validate selected fields
    if selected_fields is None:
        fields = all_fields
    else:
        fields = [f for f in selected_fields if f in all_fields]
        if not fields:
            raise ValueError("No valid fields selected")
    
    # Create identifier for Index
    id = identifier.Identifier(
        asset_type=asset.AssetType.ASSET_TYPE_INDEX,
        ticker=ticker
    )
    id.provider.value = provider
    
    # Build constraints only if we have at least one date (no time intervals due to server limitation)
    constraints_obj = None
    if start_date is not None or end_date is not None:
        # Create DateInterval with only the dates that are provided
        date_interval_kwargs = {}
        if start_date is not None:
            date_interval_kwargs['start_date'] = _parse_date_input(start_date)
        if end_date is not None:
            date_interval_kwargs['end_date'] = _parse_date_input(end_date)
            
        constraints_obj = constraints.Constraints(
            date_intervals=[date_interval.DateInterval(**date_interval_kwargs)]
        )

    # Create request with or without constraints
    request_kwargs = {
        'identifier': id,
        'fields': fields
    }

    if constraints_obj is not None:
        request_kwargs['constraints'] = constraints_obj
        
    try:
        # Open gRPC channel
        with channel_helpers.get_grpc_channel() as channel:
            # Send request and receive response
            token = token_helpers.get_token()
            first = True
            response = []
            info = None
            # Create service stub for Tick service
            service = get_tick_service.TickServiceStub(channel)
            scalar_request = get_tick.TickRequest(**request_kwargs)
            
            for data in service.TickScalarStream(request=scalar_request, metadata=[('authorization', token)]):
                if first:
                    info = data
                    first = False
                else:
                    response.append(data.data)

        # Process the response
        if not response or info is None:
            print("No data received")
            return pd.DataFrame()

        # Get field indices
        available_fields = [f for f in info.info.fields]
        field_indices = {field: available_fields.index(field)
                        for field in fields if field in available_fields}

        # Extract timestamps with full precision (including microseconds if available)
        dates = []
        for d in response:
            dt = datetime(d.datetime.year, d.datetime.month, d.datetime.day, 
                         d.datetime.hours, d.datetime.minutes, d.datetime.seconds)
            # Add microseconds if available in the protobuf message
            if hasattr(d.datetime, 'nanos'):
                # Convert nanoseconds to microseconds (Python datetime only supports microseconds)
                microseconds = d.datetime.nanos // 1000
                dt = dt.replace(microsecond=microseconds)
            elif hasattr(d.datetime, 'micros'):
                dt = dt.replace(microsecond=d.datetime.micros)
            dates.append(dt)

        # Create dictionary for DataFrame
        data_dict = {}
        
        # Extract data for each field
        for field_name, field_index in field_indices.items():
            data_dict[field_name] = [b.data[field_index] for b in response]

        # Create DataFrame
        df = pd.DataFrame(data_dict, index=dates)
        df.index.name = 'Datetime'

        # Sort by date for better readability
        df = df.sort_index()
        
        # Apply client-side time filtering if needed
        if not df.empty and (start_time is not None or end_time is not None):
            
            # Convert string times to time objects if needed
            if isinstance(start_time, str):
                time_parts = start_time.split(':')
                hour = int(time_parts[0])
                minute = int(time_parts[1]) if len(time_parts) > 1 else 0
                start_time_obj = datetime.min.time().replace(hour=hour, minute=minute)
            else:
                start_time_obj = start_time
            
            if isinstance(end_time, str):
                time_parts = end_time.split(':')
                hour = int(time_parts[0])
                minute = int(time_parts[1]) if len(time_parts) > 1 else 0
                end_time_obj = datetime.min.time().replace(hour=hour, minute=minute)
            else:
                end_time_obj = end_time
            
            # Apply time filtering
            if start_time_obj is not None and end_time_obj is not None:
                df = df.between_time(start_time_obj, end_time_obj)
            elif start_time_obj is not None:
                df = df[df.index.time >= start_time_obj]
            elif end_time_obj is not None:
                df = df[df.index.time <= end_time_obj]

        return df
    
    except grpc.RpcError as e:
        print(f"gRPC Error: {e.code().name}")
        print(f"Details: {e.details()}")
        return pd.DataFrame()
    except Exception as e:
        print(f"Error: {str(e)}")
        return pd.DataFrame()

def get_cds_index_intraday(ticker, start_date=None, end_date=None, sampling=sampling.SAMPLING_ONE_MINUTE, selected_fields=None, provider="Markit"):
    """
    Fetch CDS Index intraday data from gRPC API for a given ticker and date range.    
    
    Parameters:
    ticker (str): The ticker symbol
    start_date (datetime.date or str, optional): Start date for data retrieval.
                                                 If None, set not limits
    end_date (datetime.date or str, optional): End date for data retrieval.
                                               If None, set not limits
    sampling (sampling, optional): Sampling perdiod for intrday. Default to one minute.
    selected_fields (list, optional): List of specific fields to retrieve.
                                      If None, gets all fields.
    provider (str): Data provider, default is "Markit"
            Returns:
    pd.DataFrame: DataFrame with Date as index and all available fields as columns
    """

    # All available fields
    all_fields = [
        'BidConventionalSpread',
        'BidPrice',
        'MidConventionalSpread',
        'MidPrice',
        'AskConventionalSpread',
        'AskPrice'
    ]    
    
    # Use all fields if none specified, otherwise validate selected fields
    if selected_fields is None:
        fields = all_fields
    else:
        fields = [f for f in selected_fields if f in all_fields]
        if not fields:
            raise ValueError("No valid fields selected")
    
    # Create identifier
    id = identifier.Identifier(
        asset_type=asset.AssetType.ASSET_TYPE_CDS_INDEX,
        ticker=ticker
    )
    id.provider.value = provider
    
    # Build constraints only if we have at least one date
    constraints_obj = None
    if start_date is not None or end_date is not None:
        # Create DateInterval with only the dates that are provided
        date_interval_kwargs = {}
        if start_date is not None:
            date_interval_kwargs['start_date'] = _parse_date_input(start_date)
        if end_date is not None:
            date_interval_kwargs['end_date'] = _parse_date_input(end_date)

        constraints_obj = constraints.Constraints(
            date_intervals=[date_interval.DateInterval(**date_interval_kwargs)]
        )

    # Create request with or without constraints
    request_kwargs = {
        'identifier': id,
        'fields': fields,
        'sampling': sampling
    }

    if constraints_obj is not None:
        request_kwargs['constraints'] = constraints_obj
    try:
        # Open gRPC channel
        with channel_helpers.get_grpc_channel() as channel:
            # Send request and receive response
            token = token_helpers.get_token()
            first = True
            response = []
            info = None
            # Create service stub
            service = get_intraday_service.IntradayServiceStub(channel)
            scalar_request = get_intraday.IntradayRequest(**request_kwargs)
            
            for data in service.IntradayScalarStream(request=scalar_request, metadata=[('authorization', token)]):
                if first:
                    info = data
                    first = False
                else:
                    response.append(data.data)

        # Process the response
        if not response or info is None:
            print("No data received")
            return pd.DataFrame()

        # Get field indices
        available_fields = [f for f in info.info.fields]
        field_indices = {field: available_fields.index(field)
                        for field in fields if field in available_fields}

        # Extract dates
        dates = [datetime(d.datetime.year, d.datetime.month, d.datetime.day, d.datetime.hours, d.datetime.minutes, d.datetime.seconds) for d in response]

        # Create dictionary for DataFrame
        data_dict = {}
        
        # Extract data for each field
        for field_name, field_index in field_indices.items():
            data_dict[field_name] = [b.data[field_index] for b in response]

        # Create DataFrame
        df = pd.DataFrame(data_dict, index=dates)
        df.index.name = 'Datetime'

        # Sort by date for better readability
        df = df.sort_index()
        return df
    
    except grpc.RpcError as e:
        print(f"gRPC Error: {e.code().name}")
        print(f"Details: {e.details()}")
        return pd.DataFrame()
    except Exception as e:
        print(f"Error: {str(e)}")
        return pd.DataFrame()

def get_cds_intraday(ticker, start_date=None, end_date=None, sampling=sampling.SAMPLING_ONE_MINUTE, selected_fields=None, provider="Markit"):
    """
    Fetch CDS intraday data from gRPC API for a given ticker and date range.    
    
    Parameters:
    ticker (str): The ticker symbol
    start_date (datetime.date or str, optional): Start date for data retrieval.
                                                 If None, set not limits
    end_date (datetime.date or str, optional): End date for data retrieval.
                                               If None, set not limits
    sampling (sampling, optional): Sampling perdiod for intrday. Default to one minute.
    selected_fields (list, optional): List of specific fields to retrieve.
                                      If None, gets all fields.
    provider (str): Data provider, default is "Markit"
            Returns:
    pd.DataFrame: DataFrame with Date as index and all available fields as columns
    """

    # All available fields for individual CDS
    all_fields = [
        'BidConventionalSpread',
        'BidParSpread',
        'BidUpfront',
        'MidConventionalSpread',
        'MidParSpread',
        'MidUpfront',
        'AskConventionalSpread',
        'AskParSpread',
        'AskUpfront'
    ]    
    
    # Use all fields if none specified, otherwise validate selected fields
    if selected_fields is None:
        fields = all_fields
    else:
        fields = [f for f in selected_fields if f in all_fields]
        if not fields:
            raise ValueError("No valid fields selected")
    
    # Create identifier for individual CDS (not index)
    id = identifier.Identifier(
        asset_type=asset.AssetType.ASSET_TYPE_CDS,
        ticker=ticker
    )
    id.provider.value = provider
    
    # Build constraints only if we have at least one date
    constraints_obj = None
    if start_date is not None or end_date is not None:
        # Create DateInterval with only the dates that are provided
        date_interval_kwargs = {}
        if start_date is not None:
            date_interval_kwargs['start_date'] = _parse_date_input(start_date)
        if end_date is not None:
            date_interval_kwargs['end_date'] = _parse_date_input(end_date)
            
        constraints_obj = constraints.Constraints(
            date_intervals=[date_interval.DateInterval(**date_interval_kwargs)]
        )

    # Create request with or without constraints
    request_kwargs = {
        'identifier': id,
        'fields': fields,
        'sampling': sampling
    }

    if constraints_obj is not None:
        request_kwargs['constraints'] = constraints_obj
    try:
        # Open gRPC channel
        with channel_helpers.get_grpc_channel() as channel:
            # Send request and receive response
            token = token_helpers.get_token()
            first = True
            response = []
            info = None
            # Create service stub
            service = get_intraday_service.IntradayServiceStub(channel)
            scalar_request = get_intraday.IntradayRequest(**request_kwargs)
            
            for data in service.IntradayScalarStream(request=scalar_request, metadata=[('authorization', token)]):
                if first:
                    info = data
                    first = False
                else:
                    response.append(data.data)

        # Process the response
        if not response or info is None:
            print("No data received")
            return pd.DataFrame()

        # Get field indices
        available_fields = [f for f in info.info.fields]
        field_indices = {field: available_fields.index(field)
                        for field in fields if field in available_fields}

        # Extract dates
        dates = [datetime(d.datetime.year, d.datetime.month, d.datetime.day, d.datetime.hours, d.datetime.minutes, d.datetime.seconds) for d in response]

        # Create dictionary for DataFrame
        data_dict = {}
        
        # Extract data for each field
        for field_name, field_index in field_indices.items():
            data_dict[field_name] = [b.data[field_index] for b in response]

        # Create DataFrame
        df = pd.DataFrame(data_dict, index=dates)
        df.index.name = 'Datetime'

        # Sort by date for better readability
        df = df.sort_index()
        return df
    
    except grpc.RpcError as e:
        print(f"gRPC Error: {e.code().name}")
        print(f"Details: {e.details()}")
        return pd.DataFrame()
    except Exception as e:
        print(f"Error: {str(e)}")
        return pd.DataFrame()


def get_future_daily(ticker, start_date=None, end_date=None, provider="FirstRateData"):
    """
    Fetch Future daily data from gRPC API for a given ticker and optionally filter by date range.
    
    Parameters:
    ticker (str): The ticker symbol
    start_date (datetime.date or str, optional): Start date for data retrieval (format: '2025-05-28'). 
                                                 If None, no start limit is applied
    end_date (datetime.date or str, optional): End date for data retrieval (format: '2025-05-28').
                                               If None, no end limit is applied
    provider (str): Data provider, default is "FirstRateData"

    # Example usage:
    # df = get_future_daily('CL1 Comdty')  # Get all available data
    # df = get_future_daily('CL1 Comdty', start_date='2024-01-01')  # From Jan 1, 2024 onwards  
    # df = get_future_daily('CL1 Comdty', end_date='2024-12-31')    # Up to Dec 31, 2024
    # df = get_future_daily('CL1 Comdty', start_date='2024-01-01', end_date='2024-12-31')  # Full year 2024
    
    Returns:
    pd.DataFrame: DataFrame with Date as index and all available fields as columns
    """
    
    def _parse_date_for_filtering(date_input):
        """Parse date input for DataFrame filtering (returns date object, not Google date)"""
        if date_input is None:
            return None
        if isinstance(date_input, date):
            return date_input
        if isinstance(date_input, datetime):
            return date_input.date()
        if isinstance(date_input, str):
            return datetime.strptime(date_input, '%Y-%m-%d').date()
        raise ValueError(f"Invalid date type: {type(date_input)}")
    
    id = identifier.Identifier(
        ticker=ticker, 
        asset_type=asset.AssetType.ASSET_TYPE_FUTURE
    )
    id.provider.value = provider
    
    request = daily_bars.DailyBarsRequest(identifier=id)

    try:
        # Open gRPC channel
        with channel_helpers.get_grpc_channel() as channel:
            # Send request and receive response
            token = token_helpers.get_token()

            # Create service stub
            service = daily_bars_service.DailyBarsServiceStub(channel)
            response = service.DailyBars(request=request, metadata=[('authorization', token)])

        # Process the response
        if not response or not response.data:
            print("No data received")
            return pd.DataFrame()
        
        dates = [datetime(b.date.year, b.date.month, b.date.day) for b in response.data]
        opens = [b.open for b in response.data]
        highs = [b.high for b in response.data]
        lows = [b.low for b in response.data]
        closes = [b.close for b in response.data]
        volumes = [b.volume for b in response.data]

        data_dict = {'Date': dates, 'Open': opens, 'High': highs, 'Low': lows, 'Close': closes, 'Volume': volumes}
        df = pd.DataFrame(data=data_dict)
        df = df.set_index('Date')
        
        # Sort by date for better readability
        df = df.sort_index()
        
        # Apply date filtering if specified
        if start_date is not None or end_date is not None:
            # Parse date inputs
            if start_date is not None:
                start_date_parsed = _parse_date_for_filtering(start_date)
                start_datetime = datetime.combine(start_date_parsed, datetime.min.time())
            
            if end_date is not None:
                end_date_parsed = _parse_date_for_filtering(end_date)
                end_datetime = datetime.combine(end_date_parsed, datetime.max.time())
            
            # Filter the DataFrame
            if start_date is not None and end_date is not None:
                df = df[(df.index >= start_datetime) & (df.index <= end_datetime)]
            elif start_date is not None:
                df = df[df.index >= start_datetime]
            elif end_date is not None:
                df = df[df.index <= end_datetime]
        
        return df
        
    except grpc.RpcError as e:
        print(f"gRPC Error: {e.code().name}")
        print(f"Details: {e.details()}")
        return pd.DataFrame()
    except Exception as e:
        print(f"Error: {str(e)}")
        return pd.DataFrame()

def get_equity_daily(ticker, start_date=None, end_date=None, provider="FirstRateData"):
    """
    Fetch Equity daily data from gRPC API for a given ticker and optionally filter by date range.
    
    Parameters:
    ticker (str): The ticker symbol
    start_date (datetime.date or str, optional): Start date for data retrieval (format: '2025-05-28'). 
                                                 If None, no start limit is applied
    end_date (datetime.date or str, optional): End date for data retrieval (format: '2025-05-28').
                                               If None, no end limit is applied
    provider (str): Data provider, default is "FirstRateData"

    # Example usage:
    # df = get_equity_daily('AAPL US Equity')  # Get all available data
    # df = get_equity_daily('AAPL US Equity', start_date='2024-01-01')  # From Jan 1, 2024 onwards  
    # df = get_equity_daily('AAPL US Equity', end_date='2024-12-31')    # Up to Dec 31, 2024
    # df = get_equity_daily('AAPL US Equity', start_date='2024-01-01', end_date='2024-12-31')  # Full year 2024
    
    Returns:
    pd.DataFrame: DataFrame with Date as index and all available fields as columns
    """
    
    def _parse_date_for_filtering(date_input):
        """Parse date input for DataFrame filtering (returns date object, not Google date)"""
        if date_input is None:
            return None
        if isinstance(date_input, date):
            return date_input
        if isinstance(date_input, datetime):
            return date_input.date()
        if isinstance(date_input, str):
            return datetime.strptime(date_input, '%Y-%m-%d').date()
        raise ValueError(f"Invalid date type: {type(date_input)}")
    
    id = identifier.Identifier(
        ticker=ticker, 
        asset_type=asset.AssetType.ASSET_TYPE_EQUITY
    )
    id.provider.value = provider
    
    request = daily_bars.DailyBarsRequest(identifier=id)

    try:
        # Open gRPC channel
        with channel_helpers.get_grpc_channel() as channel:
            # Send request and receive response
            token = token_helpers.get_token()

            # Create service stub
            service = daily_bars_service.DailyBarsServiceStub(channel)
            response = service.DailyBars(request=request, metadata=[('authorization', token)])

        # Process the response
        if not response or not response.data:
            print("No data received")
            return pd.DataFrame()
        
        dates = [datetime(b.date.year, b.date.month, b.date.day) for b in response.data]
        opens = [b.open for b in response.data]
        highs = [b.high for b in response.data]
        lows = [b.low for b in response.data]
        closes = [b.close for b in response.data]
        volumes = [b.volume for b in response.data]

        data_dict = {'Date': dates, 'Open': opens, 'High': highs, 'Low': lows, 'Close': closes, 'Volume': volumes}
        df = pd.DataFrame(data=data_dict)
        df = df.set_index('Date')
        
        # Sort by date for better readability
        df = df.sort_index()
        
        # Apply date filtering if specified
        if start_date is not None or end_date is not None:
            # Parse date inputs
            if start_date is not None:
                start_date_parsed = _parse_date_for_filtering(start_date)
                start_datetime = datetime.combine(start_date_parsed, datetime.min.time())
            
            if end_date is not None:
                end_date_parsed = _parse_date_for_filtering(end_date)
                end_datetime = datetime.combine(end_date_parsed, datetime.max.time())
            
            # Filter the DataFrame
            if start_date is not None and end_date is not None:
                df = df[(df.index >= start_datetime) & (df.index <= end_datetime)]
            elif start_date is not None:
                df = df[df.index >= start_datetime]
            elif end_date is not None:
                df = df[df.index <= end_datetime]
        
        return df
        
    except grpc.RpcError as e:
        print(f"gRPC Error: {e.code().name}")
        print(f"Details: {e.details()}")
        return pd.DataFrame()
    except Exception as e:
        print(f"Error: {str(e)}")
        return pd.DataFrame()

def _python_date_to_google_date(py_date):
    """Convert Python date to Google Date protobuf message"""
    return date_pb2.Date(year=py_date.year, month=py_date.month, day=py_date.day)

# Helper function to parse date strings
def _parse_date_input(date_input):
    """Convert string dates to date objects if needed."""
    if date_input is None:
        return None
    if isinstance(date_input, date):
        return _python_date_to_google_date(date_input)
    if isinstance(date_input, datetime):
        return _python_date_to_google_date(date_input.date())
    if isinstance(date_input, str):
        d = datetime.strptime(date_input, '%Y-%m-%d').date()
        return _python_date_to_google_date(d)
    raise ValueError(f"Invalid date type: {type(date_input)}")


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/daily/v1/daily_market_data.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.type import date_pb2 as google_dot_type_dot_date__pb2
from systemathics.apis.type.shared.v1 import identifier_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_identifier__pb2
from systemathics.apis.type.shared.v1 import date_interval_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_date__interval__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;systemathics/apis/services/daily/v1/daily_market_data.proto\x12#systemathics.apis.services.daily.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x16google/type/date.proto\x1a\x31systemathics/apis/type/shared/v1/identifier.proto\x1a\x34systemathics/apis/type/shared/v1/date_interval.proto\"\xbb\x01\n\x16\x44\x61ilyMarketDataRequest\x12L\n\nidentifier\x18\x01 \x01(\x0b\x32,.systemathics.apis.type.shared.v1.IdentifierR\nidentifier\x12S\n\rdate_interval\x18\x02 \x01(\x0b\x32..systemathics.apis.type.shared.v1.DateIntervalR\x0c\x64\x61teInterval\"c\n\x17\x44\x61ilyMarketDataResponse\x12H\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x34.systemathics.apis.services.daily.v1.DailyMarketDataR\x04\x64\x61ta\"\xed\x11\n\x0f\x44\x61ilyMarketData\x12%\n\x04\x64\x61te\x18\x01 \x01(\x0b\x32\x11.google.type.DateR\x04\x64\x61te\x12\x12\n\x04open\x18\x02 \x01(\x01R\x04open\x12\x12\n\x04high\x18\x03 \x01(\x01R\x04high\x12\x10\n\x03low\x18\x04 \x01(\x01R\x03low\x12\x14\n\x05\x63lose\x18\x05 \x01(\x01R\x05\x63lose\x12\x16\n\x06volume\x18\x06 \x01(\x01R\x06volume\x12\x1b\n\tbid_price\x18\x07 \x01(\x01R\x08\x62idPrice\x12\x19\n\x08\x62id_size\x18\x08 \x01(\x01R\x07\x62idSize\x12\x1b\n\task_price\x18\t \x01(\x01R\x08\x61skPrice\x12\x19\n\x08\x61sk_size\x18\n \x01(\x01R\x07\x61skSize\x12\x14\n\x05\x64\x65lta\x18\x0b \x01(\x01R\x05\x64\x65lta\x12\x14\n\x05gamma\x18\x0c \x01(\x01R\x05gamma\x12\x14\n\x05theta\x18\r \x01(\x01R\x05theta\x12\x12\n\x04vega\x18\x0e \x01(\x01R\x04vega\x12\x10\n\x03rho\x18\x0f \x01(\x01R\x03rho\x12-\n\x12implied_volatility\x18\x10 \x01(\x01R\x11impliedVolatility\x12#\n\ropen_interest\x18\x11 \x01(\x01R\x0copenInterest\x12\x1e\n\nunderlying\x18\x12 \x01(\x01R\nunderlying\x12;\n\x1aimplied_volatility_30_call\x18\x13 \x01(\x01R\x17impliedVolatility30Call\x12\x39\n\x19implied_volatility_30_put\x18\x14 \x01(\x01R\x16impliedVolatility30Put\x12;\n\x1aimplied_volatility_30_mean\x18\x15 \x01(\x01R\x17impliedVolatility30Mean\x12;\n\x1aimplied_volatility_60_call\x18\x16 \x01(\x01R\x17impliedVolatility60Call\x12\x39\n\x19implied_volatility_60_put\x18\x17 \x01(\x01R\x16impliedVolatility60Put\x12;\n\x1aimplied_volatility_60_mean\x18\x18 \x01(\x01R\x17impliedVolatility60Mean\x12;\n\x1aimplied_volatility_90_call\x18\x19 \x01(\x01R\x17impliedVolatility90Call\x12\x39\n\x19implied_volatility_90_put\x18\x1a \x01(\x01R\x16impliedVolatility90Put\x12;\n\x1aimplied_volatility_90_mean\x18\x1b \x01(\x01R\x17impliedVolatility90Mean\x12=\n\x1bimplied_volatility_120_call\x18\x1c \x01(\x01R\x18impliedVolatility120Call\x12;\n\x1aimplied_volatility_120_put\x18\x1d \x01(\x01R\x17impliedVolatility120Put\x12=\n\x1bimplied_volatility_120_mean\x18\x1e \x01(\x01R\x18impliedVolatility120Mean\x12=\n\x1bimplied_volatility_150_call\x18\x1f \x01(\x01R\x18impliedVolatility150Call\x12;\n\x1aimplied_volatility_150_put\x18  \x01(\x01R\x17impliedVolatility150Put\x12=\n\x1bimplied_volatility_150_mean\x18! \x01(\x01R\x18impliedVolatility150Mean\x12=\n\x1bimplied_volatility_180_call\x18\" \x01(\x01R\x18impliedVolatility180Call\x12;\n\x1aimplied_volatility_180_put\x18# \x01(\x01R\x17impliedVolatility180Put\x12=\n\x1bimplied_volatility_180_mean\x18$ \x01(\x01R\x18impliedVolatility180Mean\x12=\n\x1bimplied_volatility_360_call\x18% \x01(\x01R\x18impliedVolatility360Call\x12;\n\x1aimplied_volatility_360_put\x18& \x01(\x01R\x17impliedVolatility360Put\x12=\n\x1bimplied_volatility_360_mean\x18\' \x01(\x01R\x18impliedVolatility360Mean\x12\x1f\n\x0b\x63\x61ll_volume\x18( \x01(\x01R\ncallVolume\x12\x1d\n\nput_volume\x18) \x01(\x01R\tputVolume\x12!\n\x0ctotal_volume\x18* \x01(\x01R\x0btotalVolume\x12,\n\x12\x63\x61ll_open_interest\x18+ \x01(\x01R\x10\x63\x61llOpenInterest\x12*\n\x11put_open_interest\x18, \x01(\x01R\x0fputOpenInterest\x12.\n\x13total_open_interest\x18- \x01(\x01R\x11totalOpenInterest\x12\x14\n\x05score\x18. \x01(\x01R\x05score\x12\x34\n\x16implied_volatility_bid\x18/ \x01(\x01R\x14impliedVolatilityBid\x12\x34\n\x16implied_volatility_ask\x18\x30 \x01(\x01R\x14impliedVolatilityAsk\x12\x36\n\x17implied_volatility_mean\x18\x31 \x01(\x01R\x15impliedVolatilityMean\x12#\n\roption_symbol\x18\x32 \x01(\tR\x0coptionSymbol\x12(\n\x10t1_open_interest\x18\x33 \x01(\x01R\x0et1OpenInterest2\xc5\x01\n\x16\x44\x61ilyMarketDataService\x12\xaa\x01\n\x0f\x44\x61ilyMarketData\x12;.systemathics.apis.services.daily.v1.DailyMarketDataRequest\x1a<.systemathics.apis.services.daily.v1.DailyMarketDataResponse\"\x1c\x82\xd3\xe4\x93\x02\x16\x12\x14/v1/daily/marketdataB\xf0\x01\n\'com.systemathics.apis.services.daily.v1B\x14\x44\x61ilyMarketDataProtoP\x01\xa2\x02\x04SASD\xaa\x02#Systemathics.Apis.Services.Daily.V1\xca\x02#Systemathics\\Apis\\Services\\Daily\\V1\xe2\x02/Systemathics\\Apis\\Services\\Daily\\V1\\GPBMetadata\xea\x02\'Systemathics::Apis::Services::Daily::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'systemathics.apis.services.daily.v1.daily_market_data_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\'com.systemathics.apis.services.daily.v1B\024DailyMarketDataProtoP\001\242\002\004SASD\252\002#Systemathics.Apis.Services.Daily.V1\312\002#Systemathics\\Apis\\Services\\Daily\\V1\342\002/Systemathics\\Apis\\Services\\Daily\\V1\\GPBMetadata\352\002\'Systemathics::Apis::Services::Daily::V1'
  _globals['_DAILYMARKETDATASERVICE'].methods_by_name['DailyMarketData']._options = None
  _globals['_DAILYMARKETDATASERVICE'].methods_by_name['DailyMarketData']._serialized_options = b'\202\323\344\223\002\026\022\024/v1/daily/marketdata'
  _globals['_DAILYMARKETDATAREQUEST']._serialized_start=260
  _globals['_DAILYMARKETDATAREQUEST']._serialized_end=447
  _globals['_DAILYMARKETDATARESPONSE']._serialized_start=449
  _globals['_DAILYMARKETDATARESPONSE']._serialized_end=548
  _globals['_DAILYMARKETDATA']._serialized_start=551
  _globals['_DAILYMARKETDATA']._serialized_end=2836
  _globals['_DAILYMARKETDATASERVICE']._serialized_start=2839
  _globals['_DAILYMARKETDATASERVICE']._serialized_end=3036
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/type/shared/v1/quotes_data.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from systemathics.apis.type.shared.v1 import quote_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_quote__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2systemathics/apis/type/shared/v1/quotes_data.proto\x12 systemathics.apis.type.shared.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,systemathics/apis/type/shared/v1/quote.proto\"\xa2\x01\n\nQuotesData\x12\x18\n\x07mapping\x18\x01 \x01(\rR\x07mapping\x12\x39\n\ntime_stamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimeStamp\x12?\n\x06quotes\x18\x03 \x03(\x0b\x32\'.systemathics.apis.type.shared.v1.QuoteR\x06quotesB\xdc\x01\n$com.systemathics.apis.type.shared.v1B\x0fQuotesDataProtoP\x01\xa2\x02\x04SATS\xaa\x02 Systemathics.Apis.Type.Shared.V1\xca\x02 Systemathics\\Apis\\Type\\Shared\\V1\xe2\x02,Systemathics\\Apis\\Type\\Shared\\V1\\GPBMetadata\xea\x02$Systemathics::Apis::Type::Shared::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'systemathics.apis.type.shared.v1.quotes_data_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n$com.systemathics.apis.type.shared.v1B\017QuotesDataProtoP\001\242\002\004SATS\252\002 Systemathics.Apis.Type.Shared.V1\312\002 Systemathics\\Apis\\Type\\Shared\\V1\342\002,Systemathics\\Apis\\Type\\Shared\\V1\\GPBMetadata\352\002$Systemathics::Apis::Type::Shared::V1'
  _globals['_QUOTESDATA']._serialized_start=168
  _globals['_QUOTESDATA']._serialized_end=330
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from systemathics.apis.services.daily.v2 import get_daily_pb2 as systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2


class DailyServiceStub(object):
    """Called to request daily prices data. 
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.DailyScalarStream = channel.unary_stream(
                '/systemathics.apis.services.daily.v2.DailyService/DailyScalarStream',
                request_serializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyRequest.SerializeToString,
                response_deserializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyScalarStreamResponse.FromString,
                )
        self.DailyVectorStream = channel.unary_stream(
                '/systemathics.apis.services.daily.v2.DailyService/DailyVectorStream',
                request_serializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyRequest.SerializeToString,
                response_deserializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyVectorStreamResponse.FromString,
                )
        self.DailyVectorKeyStream = channel.unary_stream(
                '/systemathics.apis.services.daily.v2.DailyService/DailyVectorKeyStream',
                request_serializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyVectorKeyRequest.SerializeToString,
                response_deserializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyVectorStreamResponse.FromString,
                )
        self.DailyOptionUnderlierStream = channel.unary_stream(
                '/systemathics.apis.services.daily.v2.DailyService/DailyOptionUnderlierStream',
                request_serializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyOptionUnderlierRequest.SerializeToString,
                response_deserializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyUnderlierOptionStream.FromString,
                )
        self.DailyFields = channel.unary_unary(
                '/systemathics.apis.services.daily.v2.DailyService/DailyFields',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyFieldsResponse.FromString,
                )
        self.DailyAssetProviders = channel.unary_unary(
                '/systemathics.apis.services.daily.v2.DailyService/DailyAssetProviders',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyAssetProvidersResponse.FromString,
                )


class DailyServiceServicer(object):
    """Called to request daily prices data. 
    """

    def DailyScalarStream(self, request, context):
        """Gets daily historical data timeseries by identifier using streaming
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DailyVectorStream(self, request, context):
        """Gets daily historical data timeseries by identifier using streaming
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DailyVectorKeyStream(self, request, context):
        """Gets daily historical data timeseries by identifier and key using streaming
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DailyOptionUnderlierStream(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DailyFields(self, request, context):
        """Gets all available fields by asset and provider.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DailyAssetProviders(self, request, context):
        """Gets all available provider by asset
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_DailyServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'DailyScalarStream': grpc.unary_stream_rpc_method_handler(
                    servicer.DailyScalarStream,
                    request_deserializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyRequest.FromString,
                    response_serializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyScalarStreamResponse.SerializeToString,
            ),
            'DailyVectorStream': grpc.unary_stream_rpc_method_handler(
                    servicer.DailyVectorStream,
                    request_deserializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyRequest.FromString,
                    response_serializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyVectorStreamResponse.SerializeToString,
            ),
            'DailyVectorKeyStream': grpc.unary_stream_rpc_method_handler(
                    servicer.DailyVectorKeyStream,
                    request_deserializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyVectorKeyRequest.FromString,
                    response_serializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyVectorStreamResponse.SerializeToString,
            ),
            'DailyOptionUnderlierStream': grpc.unary_stream_rpc_method_handler(
                    servicer.DailyOptionUnderlierStream,
                    request_deserializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyOptionUnderlierRequest.FromString,
                    response_serializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyUnderlierOptionStream.SerializeToString,
            ),
            'DailyFields': grpc.unary_unary_rpc_method_handler(
                    servicer.DailyFields,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyFieldsResponse.SerializeToString,
            ),
            'DailyAssetProviders': grpc.unary_unary_rpc_method_handler(
                    servicer.DailyAssetProviders,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyAssetProvidersResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'systemathics.apis.services.daily.v2.DailyService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class DailyService(object):
    """Called to request daily prices data. 
    """

    @staticmethod
    def DailyScalarStream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/systemathics.apis.services.daily.v2.DailyService/DailyScalarStream',
            systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyRequest.SerializeToString,
            systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyScalarStreamResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DailyVectorStream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/systemathics.apis.services.daily.v2.DailyService/DailyVectorStream',
            systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyRequest.SerializeToString,
            systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyVectorStreamResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DailyVectorKeyStream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/systemathics.apis.services.daily.v2.DailyService/DailyVectorKeyStream',
            systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyVectorKeyRequest.SerializeToString,
            systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyVectorStreamResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DailyOptionUnderlierStream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/systemathics.apis.services.daily.v2.DailyService/DailyOptionUnderlierStream',
            systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyOptionUnderlierRequest.SerializeToString,
            systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyUnderlierOptionStream.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DailyFields(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/systemathics.apis.services.daily.v2.DailyService/DailyFields',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyFieldsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DailyAssetProviders(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/systemathics.apis.services.daily.v2.DailyService/DailyAssetProviders',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            systemathics_dot_apis_dot_services_dot_daily_dot_v2_dot_get__daily__pb2.DailyAssetProvidersResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/static_data/v1/static_data_set.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.type import date_pb2 as google_dot_type_dot_date__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from systemathics.apis.services.static_data.v1 import static_data_pb2 as systemathics_dot_apis_dot_services_dot_static__data_dot_v1_dot_static__data__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n?systemathics/apis/services/static_data/v1/static_data_set.proto\x12)systemathics.apis.services.static_data.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x16google/type/date.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a;systemathics/apis/services/static_data/v1/static_data.proto\"\x8a\x03\n\x1dStaticDataOtcStreamingRequest\x12\x16\n\x06ticker\x18\x01 \x01(\tR\x06ticker\x12\x1a\n\x08provider\x18\x02 \x01(\tR\x08provider\x12\x16\n\x06strike\x18\x03 \x01(\x01R\x06strike\x12\\\n\rswaption_type\x18\x04 \x01(\x0e\x32\x37.systemathics.apis.services.static_data.v1.SwaptionTypeR\x0cswaptionType\x12)\n\x06\x65xpiry\x18\x05 \x01(\x0b\x32\x11.google.type.DateR\x06\x65xpiry\x12+\n\x11underlying_ticker\x18\x06 \x01(\tR\x10underlyingTicker\x12\x1a\n\x08\x63urrency\x18\x07 \x01(\tR\x08\x63urrency\x12\x1b\n\tmarkit_id\x18\x08 \x01(\tR\x08markitId\x12\x12\n\x04\x66igi\x18\t \x01(\tR\x04\x66igi\x12\x1a\n\x08\x63\x61lendar\x18\n \x01(\tR\x08\x63\x61lendar2\xbf\x01\n\x14StaticDataSetService\x12\xa6\x01\n\x1c\x43reateStaticDataOtcStreaming\x12H.systemathics.apis.services.static_data.v1.StaticDataOtcStreamingRequest\x1a\x16.google.protobuf.Empty\"$\x82\xd3\xe4\x93\x02\x1e\x12\x1c/v1/staticdata/otc_streamingB\x88\x02\n-com.systemathics.apis.services.static_data.v1B\x12StaticDataSetProtoP\x01\xa2\x02\x04SASS\xaa\x02(Systemathics.Apis.Services.StaticData.V1\xca\x02(Systemathics\\Apis\\Services\\StaticData\\V1\xe2\x02\x34Systemathics\\Apis\\Services\\StaticData\\V1\\GPBMetadata\xea\x02,Systemathics::Apis::Services::StaticData::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'systemathics.apis.services.static_data.v1.static_data_set_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n-com.systemathics.apis.services.static_data.v1B\022StaticDataSetProtoP\001\242\002\004SASS\252\002(Systemathics.Apis.Services.StaticData.V1\312\002(Systemathics\\Apis\\Services\\StaticData\\V1\342\0024Systemathics\\Apis\\Services\\StaticData\\V1\\GPBMetadata\352\002,Systemathics::Apis::Services::StaticData::V1'
  _globals['_STATICDATASETSERVICE'].methods_by_name['CreateStaticDataOtcStreaming']._options = None
  _globals['_STATICDATASETSERVICE'].methods_by_name['CreateStaticDataOtcStreaming']._serialized_options = b'\202\323\344\223\002\036\022\034/v1/staticdata/otc_streaming'
  _globals['_STATICDATAOTCSTREAMINGREQUEST']._serialized_start=255
  _globals['_STATICDATAOTCSTREAMINGREQUEST']._serialized_end=649
  _globals['_STATICDATASETSERVICE']._serialized_start=652
  _globals['_STATICDATASETSERVICE']._serialized_end=843
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/calendar/v1/calendar.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from google.type import date_pb2 as google_dot_type_dot_date__pb2
from google.type import timeofday_pb2 as google_dot_type_dot_timeofday__pb2
from google.type import datetime_pb2 as google_dot_type_dot_datetime__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5systemathics/apis/services/calendar/v1/calendar.proto\x12&systemathics.apis.services.calendar.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x16google/type/date.proto\x1a\x1bgoogle/type/timeofday.proto\x1a\x1agoogle/type/datetime.proto\x1a\x1bgoogle/protobuf/empty.proto\"p\n\x19HolidayCodeToMicsResponse\x12S\n\x05items\x18\x01 \x03(\x0b\x32=.systemathics.apis.services.calendar.v1.HolidayCodeToMicsItemR\x05items\"N\n\x15HolidayCodeToMicsItem\x12!\n\x0choliday_code\x18\x01 \x01(\tR\x0bholidayCode\x12\x12\n\x04mics\x18\x02 \x03(\tR\x04mics\"p\n\x19MicToHolidayCodesResponse\x12S\n\x05items\x18\x01 \x03(\x0b\x32=.systemathics.apis.services.calendar.v1.MicToHolidayCodesItemR\x05items\"N\n\x15MicToHolidayCodesItem\x12\x10\n\x03mic\x18\x01 \x01(\tR\x03mic\x12#\n\rholiday_codes\x18\x02 \x03(\tR\x0cholidayCodes\"\x90\x02\n\x10TradingHoursItem\x12%\n\x04\x64\x61te\x18\x01 \x01(\x0b\x32\x11.google.type.DateR\x04\x64\x61te\x12\x16\n\x06market\x18\x02 \x01(\tR\x06market\x12\x46\n\x04type\x18\x03 \x01(\x0e\x32\x32.systemathics.apis.services.calendar.v1.MarketTypeR\x04type\x12!\n\x0choliday_code\x18\x04 \x01(\tR\x0bholidayCode\x12R\n\x05hours\x18\x05 \x03(\x0b\x32<.systemathics.apis.services.calendar.v1.TradingHoursActivityR\x05hours\"\xb2\x06\n\x14TradingHoursActivity\x12\x1a\n\x08\x61\x63tivity\x18\x01 \x01(\tR\x08\x61\x63tivity\x12\x1c\n\tfunctions\x18\x02 \x03(\tR\tfunctions\x12O\n\tirregular\x18\x03 \x01(\x0e\x32\x31.systemathics.apis.services.calendar.v1.IrregularR\tirregular\x12\x1b\n\tnew_hours\x18\x04 \x01(\x08R\x08newHours\x12>\n\x0flocal_open_time\x18\x05 \x01(\x0b\x32\x16.google.type.TimeOfDayR\rlocalOpenTime\x12@\n\x10local_close_time\x18\x06 \x01(\x0b\x32\x16.google.type.TimeOfDayR\x0elocalCloseTime\x12\\\n\x1cvariable_seconds_before_open\x18\x07 \x01(\x0b\x32\x1b.google.protobuf.Int32ValueR\x19variableSecondsBeforeOpen\x12Z\n\x1bvariable_seconds_after_open\x18\x08 \x01(\x0b\x32\x1b.google.protobuf.Int32ValueR\x18variableSecondsAfterOpen\x12^\n\x1dvariable_seconds_before_close\x18\t \x01(\x0b\x32\x1b.google.protobuf.Int32ValueR\x1avariableSecondsBeforeClose\x12\\\n\x1cvariable_seconds_after_close\x18\n \x01(\x0b\x32\x1b.google.protobuf.Int32ValueR\x19variableSecondsAfterClose\x12:\n\rutc_open_time\x18\x0b \x01(\x0b\x32\x16.google.type.TimeOfDayR\x0butcOpenTime\x12<\n\x0eutc_close_time\x18\x0c \x01(\x0b\x32\x16.google.type.TimeOfDayR\x0cutcCloseTime\"f\n\x14TradingHoursResponse\x12N\n\x05items\x18\x01 \x03(\x0b\x32\x38.systemathics.apis.services.calendar.v1.TradingHoursItemR\x05items\"\'\n\x13TradingHoursRequest\x12\x10\n\x03mic\x18\x01 \x01(\tR\x03mic\"-\n\x17TradingHoursMicResponse\x12\x12\n\x04mics\x18\x01 \x03(\tR\x04mics\"2\n\x14HolidayCenterRequest\x12\x1a\n\x08\x63urrency\x18\x01 \x01(\tR\x08\x63urrency\";\n\x18HolidaysAsOfDatesRequest\x12\x1f\n\x0b\x63\x65nter_code\x18\x01 \x01(\tR\ncenterCode\"g\n\x0fHolidaysRequest\x12\x1f\n\x0b\x63\x65nter_code\x18\x01 \x01(\tR\ncenterCode\x12\x33\n\nas_of_date\x18\x02 \x01(\x0b\x32\x15.google.type.DateTimeR\x08\x61sOfDate\"h\n\x15HolidayCenterResponse\x12O\n\x05items\x18\x01 \x03(\x0b\x32\x39.systemathics.apis.services.calendar.v1.HolidayCenterItemR\x05items\"\x9e\x02\n\x11HolidayCenterItem\x12\x12\n\x04\x63ode\x18\x01 \x01(\tR\x04\x63ode\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12\x18\n\x07\x63ountry\x18\x03 \x01(\tR\x07\x63ountry\x12\x1a\n\x08\x63urrency\x18\x04 \x01(\tR\x08\x63urrency\x12V\n\x0choliday_type\x18\x05 \x01(\x0e\x32\x33.systemathics.apis.services.calendar.v1.HolidayTypeR\x0bholidayType\x12\x1f\n\x0biso_country\x18\x06 \x01(\tR\nisoCountry\x12\x1e\n\ndefinition\x18\x07 \x01(\tR\ndefinition\x12\x12\n\x04mics\x18\x08 \x03(\tR\x04mics\";\n\x19HolidayCurrenciesResponse\x12\x1e\n\ncurrencies\x18\x01 \x03(\tR\ncurrencies\"L\n\x0bHolidayItem\x12%\n\x04\x64\x61te\x18\x01 \x01(\x0b\x32\x11.google.type.DateR\x04\x64\x61te\x12\x16\n\x06reason\x18\x02 \x01(\tR\x06reason\"R\n\x19HolidaysAsOfDatesResponse\x12\x35\n\x0b\x61s_of_dates\x18\x01 \x03(\x0b\x32\x15.google.type.DateTimeR\tasOfDates\"\xc4\x01\n\x10HolidaysResponse\x12Z\n\x0b\x63\x65nter_info\x18\x01 \x01(\x0b\x32\x39.systemathics.apis.services.calendar.v1.HolidayCenterItemR\ncenterInfo\x12T\n\x0b\x63losed_days\x18\x02 \x03(\x0b\x32\x33.systemathics.apis.services.calendar.v1.HolidayItemR\nclosedDays*z\n\tIrregular\x12\x19\n\x15IRREGULAR_UNSPECIFIED\x10\x00\x12\x11\n\rIRREGULAR_YES\x10\x01\x12\x10\n\x0cIRREGULAR_NO\x10\x02\x12\x16\n\x12IRREGULAR_WEEK_END\x10\x03\x12\x15\n\x11IRREGULAR_HOLIDAY\x10\x04*\xe9\x01\n\nMarketType\x12\x1b\n\x17MARKET_TYPE_UNSPECIFIED\x10\x00\x12\x1b\n\x17MARKET_TYPE_CASH_EQUITY\x10\x01\x12 \n\x1cMARKET_TYPE_FUTURES_CONTRACT\x10\x02\x12\"\n\x1eMARKET_TYPE_OPTIONS_ON_FUTURES\x10\x03\x12%\n!MARKET_TYPE_OPTIONS_ON_SECURITIES\x10\x04\x12\x1d\n\x19MARKET_TYPE_OTHER_OPTIONS\x10\x05\x12\x15\n\x11MARKET_TYPE_OTHER\x10\x06*\xd1\x01\n\x0bHolidayType\x12\x1c\n\x18HOLIDAY_TYPE_UNSPECIFIED\x10\x00\x12\x15\n\x11HOLIDAY_TYPE_BANK\x10\x01\x12 \n\x1cHOLIDAY_TYPE_FUTURES_TRADING\x10\x02\x12\x16\n\x12HOLIDAY_TYPE_OTHER\x10\x03\x12*\n&HOLIDAY_TYPE_STOCK_EXCHANGE_SETTLEMENT\x10\x04\x12\'\n#HOLIDAY_TYPE_STOCK_EXCHANGE_TRADING\x10\x05\x32\xc2\n\n\x0f\x43\x61lendarService\x12\x98\x01\n\x11HolidayCurrencies\x12\x16.google.protobuf.Empty\x1a\x41.systemathics.apis.services.calendar.v1.HolidayCurrenciesResponse\"(\x82\xd3\xe4\x93\x02\"\x12 /v1/calendar/holidays/currencies\x12\xb2\x01\n\rHolidayCenter\x12<.systemathics.apis.services.calendar.v1.HolidayCenterRequest\x1a=.systemathics.apis.services.calendar.v1.HolidayCenterResponse\"$\x82\xd3\xe4\x93\x02\x1e\x12\x1c/v1/calendar/holidays/center\x12\xc3\x01\n\x11HolidaysAsOfDates\x12@.systemathics.apis.services.calendar.v1.HolidaysAsOfDatesRequest\x1a\x41.systemathics.apis.services.calendar.v1.HolidaysAsOfDatesResponse\")\x82\xd3\xe4\x93\x02#\x12!/v1/calendar/holidays_as_of_dates\x12\x9c\x01\n\x08Holidays\x12\x37.systemathics.apis.services.calendar.v1.HolidaysRequest\x1a\x38.systemathics.apis.services.calendar.v1.HolidaysResponse\"\x1d\x82\xd3\xe4\x93\x02\x17\x12\x15/v1/calendar/holidays\x12\x93\x01\n\x0fTradingHoursMic\x12\x16.google.protobuf.Empty\x1a?.systemathics.apis.services.calendar.v1.TradingHoursMicResponse\"\'\x82\xd3\xe4\x93\x02!\x12\x1f/v1/calendar/trading_hours/mics\x12\xad\x01\n\x0cTradingHours\x12;.systemathics.apis.services.calendar.v1.TradingHoursRequest\x1a<.systemathics.apis.services.calendar.v1.TradingHoursResponse\"\"\x82\xd3\xe4\x93\x02\x1c\x12\x1a/v1/calendar/trading_hours\x12\x98\x01\n\x11HolidayCodeToMics\x12\x16.google.protobuf.Empty\x1a\x41.systemathics.apis.services.calendar.v1.HolidayCodeToMicsResponse\"(\x82\xd3\xe4\x93\x02\"\x12 /v1/calendar/holiday_code_2_mics\x12\x98\x01\n\x11MicToHolidayCodes\x12\x16.google.protobuf.Empty\x1a\x41.systemathics.apis.services.calendar.v1.MicToHolidayCodesResponse\"(\x82\xd3\xe4\x93\x02\"\x12 /v1/calendar/mic_2_holiday_codesB\xf8\x01\n*com.systemathics.apis.services.calendar.v1B\rCalendarProtoP\x01\xa2\x02\x04SASC\xaa\x02&Systemathics.Apis.Services.Calendar.V1\xca\x02&Systemathics\\Apis\\Services\\Calendar\\V1\xe2\x02\x32Systemathics\\Apis\\Services\\Calendar\\V1\\GPBMetadata\xea\x02*Systemathics::Apis::Services::Calendar::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'systemathics.apis.services.calendar.v1.calendar_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n*com.systemathics.apis.services.calendar.v1B\rCalendarProtoP\001\242\002\004SASC\252\002&Systemathics.Apis.Services.Calendar.V1\312\002&Systemathics\\Apis\\Services\\Calendar\\V1\342\0022Systemathics\\Apis\\Services\\Calendar\\V1\\GPBMetadata\352\002*Systemathics::Apis::Services::Calendar::V1'
  _globals['_CALENDARSERVICE'].methods_by_name['HolidayCurrencies']._options = None
  _globals['_CALENDARSERVICE'].methods_by_name['HolidayCurrencies']._serialized_options = b'\202\323\344\223\002\"\022 /v1/calendar/holidays/currencies'
  _globals['_CALENDARSERVICE'].methods_by_name['HolidayCenter']._options = None
  _globals['_CALENDARSERVICE'].methods_by_name['HolidayCenter']._serialized_options = b'\202\323\344\223\002\036\022\034/v1/calendar/holidays/center'
  _globals['_CALENDARSERVICE'].methods_by_name['HolidaysAsOfDates']._options = None
  _globals['_CALENDARSERVICE'].methods_by_name['HolidaysAsOfDates']._serialized_options = b'\202\323\344\223\002#\022!/v1/calendar/holidays_as_of_dates'
  _globals['_CALENDARSERVICE'].methods_by_name['Holidays']._options = None
  _globals['_CALENDARSERVICE'].methods_by_name['Holidays']._serialized_options = b'\202\323\344\223\002\027\022\025/v1/calendar/holidays'
  _globals['_CALENDARSERVICE'].methods_by_name['TradingHoursMic']._options = None
  _globals['_CALENDARSERVICE'].methods_by_name['TradingHoursMic']._serialized_options = b'\202\323\344\223\002!\022\037/v1/calendar/trading_hours/mics'
  _globals['_CALENDARSERVICE'].methods_by_name['TradingHours']._options = None
  _globals['_CALENDARSERVICE'].methods_by_name['TradingHours']._serialized_options = b'\202\323\344\223\002\034\022\032/v1/calendar/trading_hours'
  _globals['_CALENDARSERVICE'].methods_by_name['HolidayCodeToMics']._options = None
  _globals['_CALENDARSERVICE'].methods_by_name['HolidayCodeToMics']._serialized_options = b'\202\323\344\223\002\"\022 /v1/calendar/holiday_code_2_mics'
  _globals['_CALENDARSERVICE'].methods_by_name['MicToHolidayCodes']._options = None
  _globals['_CALENDARSERVICE'].methods_by_name['MicToHolidayCodes']._serialized_options = b'\202\323\344\223\002\"\022 /v1/calendar/mic_2_holiday_codes'
  _globals['_IRREGULAR']._serialized_start=2980
  _globals['_IRREGULAR']._serialized_end=3102
  _globals['_MARKETTYPE']._serialized_start=3105
  _globals['_MARKETTYPE']._serialized_end=3338
  _globals['_HOLIDAYTYPE']._serialized_start=3341
  _globals['_HOLIDAYTYPE']._serialized_end=3550
  _globals['_HOLIDAYCODETOMICSRESPONSE']._serialized_start=269
  _globals['_HOLIDAYCODETOMICSRESPONSE']._serialized_end=381
  _globals['_HOLIDAYCODETOMICSITEM']._serialized_start=383
  _globals['_HOLIDAYCODETOMICSITEM']._serialized_end=461
  _globals['_MICTOHOLIDAYCODESRESPONSE']._serialized_start=463
  _globals['_MICTOHOLIDAYCODESRESPONSE']._serialized_end=575
  _globals['_MICTOHOLIDAYCODESITEM']._serialized_start=577
  _globals['_MICTOHOLIDAYCODESITEM']._serialized_end=655
  _globals['_TRADINGHOURSITEM']._serialized_start=658
  _globals['_TRADINGHOURSITEM']._serialized_end=930
  _globals['_TRADINGHOURSACTIVITY']._serialized_start=933
  _globals['_TRADINGHOURSACTIVITY']._serialized_end=1751
  _globals['_TRADINGHOURSRESPONSE']._serialized_start=1753
  _globals['_TRADINGHOURSRESPONSE']._serialized_end=1855
  _globals['_TRADINGHOURSREQUEST']._serialized_start=1857
  _globals['_TRADINGHOURSREQUEST']._serialized_end=1896
  _globals['_TRADINGHOURSMICRESPONSE']._serialized_start=1898
  _globals['_TRADINGHOURSMICRESPONSE']._serialized_end=1943
  _globals['_HOLIDAYCENTERREQUEST']._serialized_start=1945
  _globals['_HOLIDAYCENTERREQUEST']._serialized_end=1995
  _globals['_HOLIDAYSASOFDATESREQUEST']._serialized_start=1997
  _globals['_HOLIDAYSASOFDATESREQUEST']._serialized_end=2056
  _globals['_HOLIDAYSREQUEST']._serialized_start=2058
  _globals['_HOLIDAYSREQUEST']._serialized_end=2161
  _globals['_HOLIDAYCENTERRESPONSE']._serialized_start=2163
  _globals['_HOLIDAYCENTERRESPONSE']._serialized_end=2267
  _globals['_HOLIDAYCENTERITEM']._serialized_start=2270
  _globals['_HOLIDAYCENTERITEM']._serialized_end=2556
  _globals['_HOLIDAYCURRENCIESRESPONSE']._serialized_start=2558
  _globals['_HOLIDAYCURRENCIESRESPONSE']._serialized_end=2617
  _globals['_HOLIDAYITEM']._serialized_start=2619
  _globals['_HOLIDAYITEM']._serialized_end=2695
  _globals['_HOLIDAYSASOFDATESRESPONSE']._serialized_start=2697
  _globals['_HOLIDAYSASOFDATESRESPONSE']._serialized_end=2779
  _globals['_HOLIDAYSRESPONSE']._serialized_start=2782
  _globals['_HOLIDAYSRESPONSE']._serialized_end=2978
  _globals['_CALENDARSERVICE']._serialized_start=3553
  _globals['_CALENDARSERVICE']._serialized_end=4899
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/daily/v1/daily_market_data_by_underlying.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.type import date_pb2 as google_dot_type_dot_date__pb2
from systemathics.apis.type.shared.v1 import identifier_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_identifier__pb2
from systemathics.apis.type.shared.v1 import option_type_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_option__type__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nIsystemathics/apis/services/daily/v1/daily_market_data_by_underlying.proto\x12#systemathics.apis.services.daily.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x16google/type/date.proto\x1a\x31systemathics/apis/type/shared/v1/identifier.proto\x1a\x32systemathics/apis/type/shared/v1/option_type.proto\"\xae\x01\n\"DailyMarketDataByUnderlyingRequest\x12\x61\n\x15underlying_identifier\x18\x01 \x01(\x0b\x32,.systemathics.apis.type.shared.v1.IdentifierR\x14underlyingIdentifier\x12%\n\x04\x64\x61te\x18\x02 \x01(\x0b\x32\x11.google.type.DateR\x04\x64\x61te\"{\n#DailyMarketDataByUnderlyingResponse\x12T\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32@.systemathics.apis.services.daily.v1.DailyMarketDataByUnderlyingR\x04\x64\x61ta\"\xb8\x06\n\x1b\x44\x61ilyMarketDataByUnderlying\x12%\n\x04\x64\x61te\x18\x01 \x01(\x0b\x32\x11.google.type.DateR\x04\x64\x61te\x12+\n\x11underlying_symbol\x18\x02 \x01(\tR\x10underlyingSymbol\x12)\n\x10underlying_price\x18\x03 \x01(\x01R\x0funderlyingPrice\x12#\n\roption_symbol\x18\x04 \x01(\tR\x0coptionSymbol\x12M\n\x0boption_type\x18\x05 \x01(\x0e\x32,.systemathics.apis.type.shared.v1.OptionTypeR\noptionType\x12#\n\roption_strike\x18\x06 \x01(\x01R\x0coptionStrike\x12G\n\x16option_expiration_date\x18\x07 \x01(\x0b\x32\x11.google.type.DateR\x14optionExpirationDate\x12\x1d\n\nlast_price\x18\x08 \x01(\x01R\tlastPrice\x12\x1b\n\tbid_price\x18\t \x01(\x01R\x08\x62idPrice\x12\x1b\n\task_price\x18\n \x01(\x01R\x08\x61skPrice\x12\x16\n\x06volume\x18\x0b \x01(\x01R\x06volume\x12#\n\ropen_interest\x18\x0c \x01(\x01R\x0copenInterest\x12(\n\x10t1_open_interest\x18\r \x01(\x01R\x0et1OpenInterest\x12\x14\n\x05\x64\x65lta\x18\x0e \x01(\x01R\x05\x64\x65lta\x12\x14\n\x05gamma\x18\x0f \x01(\x01R\x05gamma\x12\x14\n\x05theta\x18\x10 \x01(\x01R\x05theta\x12\x12\n\x04vega\x18\x11 \x01(\x01R\x04vega\x12\x34\n\x16implied_volatility_bid\x18\x12 \x01(\x01R\x14impliedVolatilityBid\x12\x34\n\x16implied_volatility_ask\x18\x13 \x01(\x01R\x14impliedVolatilityAsk\x12\x36\n\x17implied_volatility_mean\x18\x14 \x01(\x01R\x15impliedVolatilityMean2\x83\x02\n\"DailyMarketDataByUnderlyingService\x12\xdc\x01\n\x1b\x44\x61ilyMarketDataByUnderlying\x12G.systemathics.apis.services.daily.v1.DailyMarketDataByUnderlyingRequest\x1aH.systemathics.apis.services.daily.v1.DailyMarketDataByUnderlyingResponse\"*\x82\xd3\xe4\x93\x02$\x12\"/v1/daily/marketdata_by_underlyingB\xfc\x01\n\'com.systemathics.apis.services.daily.v1B DailyMarketDataByUnderlyingProtoP\x01\xa2\x02\x04SASD\xaa\x02#Systemathics.Apis.Services.Daily.V1\xca\x02#Systemathics\\Apis\\Services\\Daily\\V1\xe2\x02/Systemathics\\Apis\\Services\\Daily\\V1\\GPBMetadata\xea\x02\'Systemathics::Apis::Services::Daily::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'systemathics.apis.services.daily.v1.daily_market_data_by_underlying_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\'com.systemathics.apis.services.daily.v1B DailyMarketDataByUnderlyingProtoP\001\242\002\004SASD\252\002#Systemathics.Apis.Services.Daily.V1\312\002#Systemathics\\Apis\\Services\\Daily\\V1\342\002/Systemathics\\Apis\\Services\\Daily\\V1\\GPBMetadata\352\002\'Systemathics::Apis::Services::Daily::V1'
  _globals['_DAILYMARKETDATABYUNDERLYINGSERVICE'].methods_by_name['DailyMarketDataByUnderlying']._options = None
  _globals['_DAILYMARKETDATABYUNDERLYINGSERVICE'].methods_by_name['DailyMarketDataByUnderlying']._serialized_options = b'\202\323\344\223\002$\022\"/v1/daily/marketdata_by_underlying'
  _globals['_DAILYMARKETDATABYUNDERLYINGREQUEST']._serialized_start=272
  _globals['_DAILYMARKETDATABYUNDERLYINGREQUEST']._serialized_end=446
  _globals['_DAILYMARKETDATABYUNDERLYINGRESPONSE']._serialized_start=448
  _globals['_DAILYMARKETDATABYUNDERLYINGRESPONSE']._serialized_end=571
  _globals['_DAILYMARKETDATABYUNDERLYING']._serialized_start=574
  _globals['_DAILYMARKETDATABYUNDERLYING']._serialized_end=1398
  _globals['_DAILYMARKETDATABYUNDERLYINGSERVICE']._serialized_start=1401
  _globals['_DAILYMARKETDATABYUNDERLYINGSERVICE']._serialized_end=1660
# @@protoc_insertion_point(module_scope)
